@@ -1,6 +1,6 @@
 //! Implementations of DeriveFrom, setting non-explicit fields.
 use crate::{
-    Cluster, ClusterInfo, EnumeratedValues, Field, FieldInfo, Peripheral, Register, RegisterInfo,
+    ClusterInfo, EnumeratedValues, FieldInfo, MaybeArray, PeripheralInfo, RegisterInfo,
     RegisterProperties,
 };
 
@@ -29,7 +29,7 @@ impl DeriveFrom for ClusterInfo {
 impl DeriveFrom for EnumeratedValues {
     fn derive_from(&self, other: &Self) -> Self {
         let mut derived = self.clone();
-        derived.usage = derived.usage.or_else(|| other.usage.clone());
+        derived.usage = derived.usage.or(other.usage);
         if derived.values.is_empty() {
             derived.values = other.values.clone();
         }
@@ -37,11 +37,21 @@ impl DeriveFrom for EnumeratedValues {
     }
 }
 
-impl DeriveFrom for Peripheral {
+impl DeriveFrom for PeripheralInfo {
     fn derive_from(&self, other: &Self) -> Self {
         let mut derived = self.clone();
-        derived.group_name = derived.group_name.or_else(|| other.group_name.clone());
+        derived.version = derived.version.or_else(|| other.version.clone());
         derived.description = derived.description.or_else(|| other.description.clone());
+        derived.group_name = derived.group_name.or_else(|| other.group_name.clone());
+        derived.prepend_to_name = derived
+            .prepend_to_name
+            .or_else(|| other.prepend_to_name.clone());
+        derived.append_to_name = derived
+            .append_to_name
+            .or_else(|| other.append_to_name.clone());
+        derived.header_struct_name = derived
+            .header_struct_name
+            .or_else(|| other.header_struct_name.clone());
         derived.default_register_properties = derived
             .default_register_properties
             .derive_from(&other.default_register_properties);
@@ -60,6 +70,7 @@ impl DeriveFrom for RegisterInfo {
         derived.properties = derived.properties.derive_from(&other.properties);
         derived.fields = derived.fields.or_else(|| other.fields.clone());
         derived.write_constraint = derived.write_constraint.or(other.write_constraint);
+        derived.read_action = derived.read_action.or(other.read_action);
         derived.modified_write_values = derived
             .modified_write_values
             .or(other.modified_write_values);
@@ -69,11 +80,12 @@ impl DeriveFrom for RegisterInfo {
 
 impl DeriveFrom for RegisterProperties {
     fn derive_from(&self, other: &Self) -> Self {
-        let mut derived = self.clone();
+        let mut derived = *self;
         derived.size = derived.size.or(other.size);
+        derived.access = derived.access.or(other.access);
+        derived.protection = derived.protection.or(other.protection);
         derived.reset_value = derived.reset_value.or(other.reset_value);
         derived.reset_mask = derived.reset_mask.or(other.reset_mask);
-        derived.access = derived.access.or(other.access);
         derived
     }
 }
@@ -87,6 +99,7 @@ impl DeriveFrom for FieldInfo {
             derived.enumerated_values = other.enumerated_values.clone();
         }
         derived.write_constraint = derived.write_constraint.or(other.write_constraint);
+        derived.read_action = derived.read_action.or(other.read_action);
         derived.modified_write_values = derived
             .modified_write_values
             .or(other.modified_write_values);
@@ -94,48 +107,19 @@ impl DeriveFrom for FieldInfo {
     }
 }
 
-impl DeriveFrom for Cluster {
+impl<T> DeriveFrom for MaybeArray<T>
+where
+    T: DeriveFrom,
+{
     fn derive_from(&self, other: &Self) -> Self {
         match (self, other) {
             (Self::Single(info), Self::Single(other_info)) => {
                 Self::Single(info.derive_from(other_info))
             }
             (Self::Single(info), Self::Array(other_info, other_dim)) => {
-                Self::Array(info.derive_from(other_info), other_dim.clone())
-            }
-            (Self::Array(info, dim), Self::Single(other_info))
-            | (Self::Array(info, dim), Self::Array(other_info, _)) => {
-                Self::Array(info.derive_from(other_info), dim.clone())
-            }
-        }
-    }
-}
-
-impl DeriveFrom for Register {
-    fn derive_from(&self, other: &Self) -> Self {
-        match (self, other) {
-            (Self::Single(info), Self::Single(other_info)) => {
-                Self::Single(info.derive_from(other_info))
-            }
-            (Self::Single(info), Self::Array(other_info, other_dim)) => {
-                Self::Array(info.derive_from(other_info), other_dim.clone())
-            }
-            (Self::Array(info, dim), Self::Single(other_info))
-            | (Self::Array(info, dim), Self::Array(other_info, _)) => {
-                Self::Array(info.derive_from(other_info), dim.clone())
-            }
-        }
-    }
-}
-
-impl DeriveFrom for Field {
-    fn derive_from(&self, other: &Self) -> Self {
-        match (self, other) {
-            (Self::Single(info), Self::Single(other_info)) => {
-                Self::Single(info.derive_from(other_info))
-            }
-            (Self::Single(info), Self::Array(other_info, other_dim)) => {
-                Self::Array(info.derive_from(other_info), other_dim.clone())
+                let mut dim = other_dim.clone();
+                dim.dim_name = None;
+                Self::Array(info.derive_from(other_info), dim)
             }
             (Self::Array(info, dim), Self::Single(other_info))
             | (Self::Array(info, dim), Self::Array(other_info, _)) => {
