@@ -1,7 +1,10 @@
 /// Allows specifying two different enumerated values
 /// depending whether it is to be used for a read or a write access.
-#[cfg_attr(feature = "serde", derive(serde::Deserialize, serde::Serialize))]
-#[cfg_attr(feature = "serde", serde(rename_all = "kebab-case"))]
+#[cfg_attr(
+    feature = "serde",
+    derive(serde::Deserialize, serde::Serialize),
+    serde(rename_all = "kebab-case")
+)]
 #[derive(Clone, Copy, Debug, Eq, PartialEq)]
 pub enum Usage {
     /// Read
@@ -19,7 +22,7 @@ impl Default for Usage {
 }
 
 impl Usage {
-    /// Parse a string into an [Usage] value, returning [`Option::None`] if the string is not valid.
+    /// Parse a string into an [`Usage`] value, returning [`Option::None`] if the string is not valid.
     pub fn parse_str(s: &str) -> Option<Self> {
         match s {
             "read" => Some(Self::Read),
